from __future__ import annotations

from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

if TYPE_CHECKING:
    from ..models.create_database_initial_cluster import CreateDatabaseInitialCluster


T = TypeVar("T", bound="CreateDatabase")


@_attrs_define
class CreateDatabase:
    """
    Attributes:
        name (str):
        initial_cluster (CreateDatabaseInitialCluster):
        provider (str):
        region (str):
        num_nodes (int | Unset):
        stream_type (str | Unset):
    """

    name: str
    initial_cluster: CreateDatabaseInitialCluster
    provider: str
    region: str
    num_nodes: int | Unset = UNSET
    stream_type: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        initial_cluster = self.initial_cluster.to_dict()

        provider = self.provider

        region = self.region

        num_nodes = self.num_nodes

        stream_type = self.stream_type

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "initialCluster": initial_cluster,
                "provider": provider,
                "region": region,
            }
        )
        if num_nodes is not UNSET:
            field_dict["numNodes"] = num_nodes
        if stream_type is not UNSET:
            field_dict["streamType"] = stream_type

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.create_database_initial_cluster import (
            CreateDatabaseInitialCluster,
        )

        d = dict(src_dict)
        name = d.pop("name")

        initial_cluster = CreateDatabaseInitialCluster.from_dict(
            d.pop("initialCluster")
        )

        provider = d.pop("provider")

        region = d.pop("region")

        num_nodes = d.pop("numNodes", UNSET)

        stream_type = d.pop("streamType", UNSET)

        create_database = cls(
            name=name,
            initial_cluster=initial_cluster,
            provider=provider,
            region=region,
            num_nodes=num_nodes,
            stream_type=stream_type,
        )

        create_database.additional_properties = d
        return create_database

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.status import Status
from ..types import (
    UNSET,
    Unset,
)

if TYPE_CHECKING:
    from ..models.auto_stop import AutoStop
    from ..models.cluster_settings import ClusterSettings


T = TypeVar("T", bound="Cluster")


@_attrs_define
class Cluster:
    """
    Attributes:
        status (Status):
        id (str):
        name (str):
        size (str):
        family_name (str):
        created_at (datetime.datetime):
        created_by (str):
        main_cluster (bool):
        settings (ClusterSettings):
        deleted_at (datetime.datetime | Unset):
        deleted_by (str | Unset):
        auto_stop (AutoStop | Unset):
    """

    status: Status
    id: str
    name: str
    size: str
    family_name: str
    created_at: datetime.datetime
    created_by: str
    main_cluster: bool
    settings: ClusterSettings
    deleted_at: datetime.datetime | Unset = UNSET
    deleted_by: str | Unset = UNSET
    auto_stop: AutoStop | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        status = self.status.value

        id = self.id

        name = self.name

        size = self.size

        family_name = self.family_name

        created_at = self.created_at.isoformat()

        created_by = self.created_by

        main_cluster = self.main_cluster

        settings = self.settings.to_dict()

        deleted_at: str | Unset = UNSET
        if not isinstance(self.deleted_at, Unset):
            deleted_at = self.deleted_at.isoformat()

        deleted_by = self.deleted_by

        auto_stop: dict[str, Any] | Unset = UNSET
        if not isinstance(self.auto_stop, Unset):
            auto_stop = self.auto_stop.to_dict()

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "status": status,
                "id": id,
                "name": name,
                "size": size,
                "familyName": family_name,
                "createdAt": created_at,
                "createdBy": created_by,
                "mainCluster": main_cluster,
                "settings": settings,
            }
        )
        if deleted_at is not UNSET:
            field_dict["deletedAt"] = deleted_at
        if deleted_by is not UNSET:
            field_dict["deletedBy"] = deleted_by
        if auto_stop is not UNSET:
            field_dict["autoStop"] = auto_stop

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.auto_stop import AutoStop
        from ..models.cluster_settings import ClusterSettings

        d = dict(src_dict)
        status = Status(d.pop("status"))

        id = d.pop("id")

        name = d.pop("name")

        size = d.pop("size")

        family_name = d.pop("familyName")

        created_at = isoparse(d.pop("createdAt"))

        created_by = d.pop("createdBy")

        main_cluster = d.pop("mainCluster")

        settings = ClusterSettings.from_dict(d.pop("settings"))

        _deleted_at = d.pop("deletedAt", UNSET)
        deleted_at: datetime.datetime | Unset
        if isinstance(_deleted_at, Unset):
            deleted_at = UNSET
        else:
            deleted_at = isoparse(_deleted_at)

        deleted_by = d.pop("deletedBy", UNSET)

        _auto_stop = d.pop("autoStop", UNSET)
        auto_stop: AutoStop | Unset
        if isinstance(_auto_stop, Unset):
            auto_stop = UNSET
        else:
            auto_stop = AutoStop.from_dict(_auto_stop)

        cluster = cls(
            status=status,
            id=id,
            name=name,
            size=size,
            family_name=family_name,
            created_at=created_at,
            created_by=created_by,
            main_cluster=main_cluster,
            settings=settings,
            deleted_at=deleted_at,
            deleted_by=deleted_by,
            auto_stop=auto_stop,
        )

        return cluster



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.extension_parameter_definitions import ExtensionParameterDefinitions


T = TypeVar("T", bound="ExtensionDetail")


@_attrs_define
class ExtensionDetail:
    """
    Attributes:
        id (str):
        version (str):
        parameter_definitions (list[ExtensionParameterDefinitions]):
    """

    id: str
    version: str
    parameter_definitions: list[ExtensionParameterDefinitions]

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        version = self.version

        parameter_definitions = []
        for parameter_definitions_item_data in self.parameter_definitions:
            parameter_definitions_item = parameter_definitions_item_data.to_dict()
            parameter_definitions.append(parameter_definitions_item)

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "version": version,
                "parameterDefinitions": parameter_definitions,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.extension_parameter_definitions import (
            ExtensionParameterDefinitions,
        )

        d = dict(src_dict)
        id = d.pop("id")

        version = d.pop("version")

        parameter_definitions = []
        _parameter_definitions = d.pop("parameterDefinitions")
        for parameter_definitions_item_data in _parameter_definitions:
            parameter_definitions_item = ExtensionParameterDefinitions.from_dict(
                parameter_definitions_item_data
            )

            parameter_definitions.append(parameter_definitions_item)

        extension_detail = cls(
            id=id,
            version=version,
            parameter_definitions=parameter_definitions,
        )

        return extension_detail



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

T = TypeVar("T", bound="ClusterSettings")


@_attrs_define
class ClusterSettings:
    """
    Attributes:
        offload_enabled (bool):
        offload_timeout_min (int):
    """

    offload_enabled: bool
    offload_timeout_min: int

    def to_dict(self) -> dict[str, Any]:
        offload_enabled = self.offload_enabled

        offload_timeout_min = self.offload_timeout_min

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "offloadEnabled": offload_enabled,
                "offloadTimeoutMin": offload_timeout_min,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        offload_enabled = d.pop("offloadEnabled")

        offload_timeout_min = d.pop("offloadTimeoutMin")

        cluster_settings = cls(
            offload_enabled=offload_enabled,
            offload_timeout_min=offload_timeout_min,
        )

        return cluster_settings



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="ClusterSize1")


@_attrs_define
class ClusterSize1:
    """
    Attributes:
        size (str):
        price (float):
        vcpu (float):
        ram (float):
        is_default (bool):
        name (str):
        family (str | Unset):
    """

    size: str
    price: float
    vcpu: float
    ram: float
    is_default: bool
    name: str
    family: str | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        size = self.size

        price = self.price

        vcpu = self.vcpu

        ram = self.ram

        is_default = self.is_default

        name = self.name

        family = self.family

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "size": size,
                "price": price,
                "vcpu": vcpu,
                "ram": ram,
                "isDefault": is_default,
                "name": name,
            }
        )
        if family is not UNSET:
            field_dict["family"] = family

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        size = d.pop("size")

        price = d.pop("price")

        vcpu = d.pop("vcpu")

        ram = d.pop("ram")

        is_default = d.pop("isDefault")

        name = d.pop("name")

        family = d.pop("family", UNSET)

        cluster_size_1 = cls(
            size=size,
            price=price,
            vcpu=vcpu,
            ram=ram,
            is_default=is_default,
            name=name,
            family=family,
        )

        return cluster_size_1



from enum import Enum


class ScheduleActionType0(str, Enum):
    ACTIONCLUSTERSCALE = "ActionClusterScale"
    ACTIONCLUSTERSTART = "ActionClusterStart"
    ACTIONCLUSTERSTOP = "ActionClusterStop"
    ACTIONDATABASEUPGRADE = "ActionDatabaseUpgrade"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.status import Status
from ..types import (
    UNSET,
    Unset,
)

if TYPE_CHECKING:
    from ..models.exasol_database_clusters import ExasolDatabaseClusters
    from ..models.exasol_database_integrations_item import (
        ExasolDatabaseIntegrationsItem,
    )


T = TypeVar("T", bound="ExasolDatabase")


@_attrs_define
class ExasolDatabase:
    """
    Attributes:
        status (Status):
        id (str):
        name (str):
        clusters (ExasolDatabaseClusters):
        provider (str):
        region (str):
        created_at (datetime.datetime):
        created_by (str):
        integrations (list[ExasolDatabaseIntegrationsItem] | Unset):
        deleted_by (str | Unset):
        deleted_at (datetime.datetime | Unset):
    """

    status: Status
    id: str
    name: str
    clusters: ExasolDatabaseClusters
    provider: str
    region: str
    created_at: datetime.datetime
    created_by: str
    integrations: list[ExasolDatabaseIntegrationsItem] | Unset = UNSET
    deleted_by: str | Unset = UNSET
    deleted_at: datetime.datetime | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        status = self.status.value

        id = self.id

        name = self.name

        clusters = self.clusters.to_dict()

        provider = self.provider

        region = self.region

        created_at = self.created_at.isoformat()

        created_by = self.created_by

        integrations: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.integrations, Unset):
            integrations = []
            for integrations_item_data in self.integrations:
                integrations_item = integrations_item_data.to_dict()
                integrations.append(integrations_item)

        deleted_by = self.deleted_by

        deleted_at: str | Unset = UNSET
        if not isinstance(self.deleted_at, Unset):
            deleted_at = self.deleted_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
                "id": id,
                "name": name,
                "clusters": clusters,
                "provider": provider,
                "region": region,
                "createdAt": created_at,
                "createdBy": created_by,
            }
        )
        if integrations is not UNSET:
            field_dict["integrations"] = integrations
        if deleted_by is not UNSET:
            field_dict["deletedBy"] = deleted_by
        if deleted_at is not UNSET:
            field_dict["deletedAt"] = deleted_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.exasol_database_clusters import ExasolDatabaseClusters
        from ..models.exasol_database_integrations_item import (
            ExasolDatabaseIntegrationsItem,
        )

        d = dict(src_dict)
        status = Status(d.pop("status"))

        id = d.pop("id")

        name = d.pop("name")

        clusters = ExasolDatabaseClusters.from_dict(d.pop("clusters"))

        provider = d.pop("provider")

        region = d.pop("region")

        created_at = isoparse(d.pop("createdAt"))

        created_by = d.pop("createdBy")

        _integrations = d.pop("integrations", UNSET)
        integrations: list[ExasolDatabaseIntegrationsItem] | Unset = UNSET
        if _integrations is not UNSET:
            integrations = []
            for integrations_item_data in _integrations:
                integrations_item = ExasolDatabaseIntegrationsItem.from_dict(
                    integrations_item_data
                )

                integrations.append(integrations_item)

        deleted_by = d.pop("deletedBy", UNSET)

        _deleted_at = d.pop("deletedAt", UNSET)
        deleted_at: datetime.datetime | Unset
        if isinstance(_deleted_at, Unset):
            deleted_at = UNSET
        else:
            deleted_at = isoparse(_deleted_at)

        exasol_database = cls(
            status=status,
            id=id,
            name=name,
            clusters=clusters,
            provider=provider,
            region=region,
            created_at=created_at,
            created_by=created_by,
            integrations=integrations,
            deleted_by=deleted_by,
            deleted_at=deleted_at,
        )

        exasol_database.additional_properties = d
        return exasol_database

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

T = TypeVar("T", bound="ExasolDatabaseClusters")


@_attrs_define
class ExasolDatabaseClusters:
    """
    Attributes:
        total (int):
        running (int):
    """

    total: int
    running: int

    def to_dict(self) -> dict[str, Any]:
        total = self.total

        running = self.running

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "total": total,
                "running": running,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        total = d.pop("total")

        running = d.pop("running")

        exasol_database_clusters = cls(
            total=total,
            running=running,
        )

        return exasol_database_clusters



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

from ..types import (
    UNSET,
    Unset,
)

if TYPE_CHECKING:
    from ..models.usage_cluster import UsageCluster


T = TypeVar("T", bound="UsageAdditionalPropertyItem")


@_attrs_define
class UsageAdditionalPropertyItem:
    """
    Attributes:
        id (str):
        name (str):
        clusters (list[UsageCluster]):
        used_storage (float | Unset):
    """

    id: str
    name: str
    clusters: list[UsageCluster]
    used_storage: float | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        clusters = []
        for clusters_item_data in self.clusters:
            clusters_item = clusters_item_data.to_dict()
            clusters.append(clusters_item)

        used_storage = self.used_storage

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "name": name,
                "clusters": clusters,
            }
        )
        if used_storage is not UNSET:
            field_dict["usedStorage"] = used_storage

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.usage_cluster import UsageCluster

        d = dict(src_dict)
        id = d.pop("id")

        name = d.pop("name")

        clusters = []
        _clusters = d.pop("clusters")
        for clusters_item_data in _clusters:
            clusters_item = UsageCluster.from_dict(clusters_item_data)

            clusters.append(clusters_item)

        used_storage = d.pop("usedStorage", UNSET)

        usage_additional_property_item = cls(
            id=id,
            name=name,
            clusters=clusters,
            used_storage=used_storage,
        )

        return usage_additional_property_item



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="AllowedIP")


@_attrs_define
class AllowedIP:
    """
    Attributes:
        id (str):
        name (str):
        cidr_ip (str):
        created_at (datetime.datetime):
        created_by (str):
        deleted_by (str | Unset):
        deleted_at (datetime.datetime | Unset):
    """

    id: str
    name: str
    cidr_ip: str
    created_at: datetime.datetime
    created_by: str
    deleted_by: str | Unset = UNSET
    deleted_at: datetime.datetime | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        cidr_ip = self.cidr_ip

        created_at = self.created_at.isoformat()

        created_by = self.created_by

        deleted_by = self.deleted_by

        deleted_at: str | Unset = UNSET
        if not isinstance(self.deleted_at, Unset):
            deleted_at = self.deleted_at.isoformat()

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "name": name,
                "cidrIp": cidr_ip,
                "createdAt": created_at,
                "createdBy": created_by,
            }
        )
        if deleted_by is not UNSET:
            field_dict["deletedBy"] = deleted_by
        if deleted_at is not UNSET:
            field_dict["deletedAt"] = deleted_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        name = d.pop("name")

        cidr_ip = d.pop("cidrIp")

        created_at = isoparse(d.pop("createdAt"))

        created_by = d.pop("createdBy")

        deleted_by = d.pop("deletedBy", UNSET)

        _deleted_at = d.pop("deletedAt", UNSET)
        deleted_at: datetime.datetime | Unset
        if isinstance(_deleted_at, Unset):
            deleted_at = UNSET
        else:
            deleted_at = isoparse(_deleted_at)

        allowed_ip = cls(
            id=id,
            name=name,
            cidr_ip=cidr_ip,
            created_at=created_at,
            created_by=created_by,
            deleted_by=deleted_by,
            deleted_at=deleted_at,
        )

        return allowed_ip



"""Contains all the data models used in inputs/outputs"""

from .allowed_ip import AllowedIP
from .api_error import ApiError
from .auto_stop import AutoStop
from .cluster import Cluster
from .cluster_action_scale import ClusterActionScale
from .cluster_action_start_stop import ClusterActionStartStop
from .cluster_connection import ClusterConnection
from .cluster_settings import ClusterSettings
from .cluster_settings_update import ClusterSettingsUpdate
from .cluster_size_1 import ClusterSize1
from .connection_i_ps import ConnectionIPs
from .create_allowed_ip import CreateAllowedIP
from .create_cluster import CreateCluster
from .create_database import CreateDatabase
from .create_database_initial_cluster import CreateDatabaseInitialCluster
from .create_extension_instance import CreateExtensionInstance
from .database_settings import DatabaseSettings
from .database_upgrade_info import DatabaseUpgradeInfo
from .dlhc_activate_status import DlhcActivateStatus
from .download_file import DownloadFile
from .exasol_database import ExasolDatabase
from .exasol_database_clusters import ExasolDatabaseClusters
from .exasol_database_integrations_item import ExasolDatabaseIntegrationsItem
from .extension import Extension
from .extension_detail import ExtensionDetail
from .extension_instance import ExtensionInstance
from .extension_parameter_definitions import ExtensionParameterDefinitions
from .extension_parameter_value import ExtensionParameterValue
from .extension_version import ExtensionVersion
from .file import File
from .get_usage_type import GetUsageType
from .patch_user import PatchUser
from .patch_user_databases import PatchUserDatabases
from .platform import Platform
from .region import Region
from .scale_cluster import ScaleCluster
from .schedule import Schedule
from .schedule_action_type_0 import ScheduleActionType0
from .schedule_state import ScheduleState
from .set_auto_updates_database import SetAutoUpdatesDatabase
from .status import Status
from .update_allowed_ip import UpdateAllowedIP
from .update_cluster import UpdateCluster
from .update_database import UpdateDatabase
from .update_profile import UpdateProfile
from .update_schedule_cron_rule import UpdateScheduleCronRule
from .update_schedule_state import UpdateScheduleState
from .upload_file import UploadFile
from .usage import Usage
from .usage_additional_property_item import UsageAdditionalPropertyItem
from .usage_cluster import UsageCluster
from .user import User
from .user_database import UserDatabase
from .user_role import UserRole
from .user_status import UserStatus

__all__ = (
    "AllowedIP",
    "ApiError",
    "AutoStop",
    "Cluster",
    "ClusterActionScale",
    "ClusterActionStartStop",
    "ClusterConnection",
    "ClusterSettings",
    "ClusterSettingsUpdate",
    "ClusterSize1",
    "ConnectionIPs",
    "CreateAllowedIP",
    "CreateCluster",
    "CreateDatabase",
    "CreateDatabaseInitialCluster",
    "CreateExtensionInstance",
    "DatabaseSettings",
    "DatabaseUpgradeInfo",
    "DlhcActivateStatus",
    "DownloadFile",
    "ExasolDatabase",
    "ExasolDatabaseClusters",
    "ExasolDatabaseIntegrationsItem",
    "Extension",
    "ExtensionDetail",
    "ExtensionInstance",
    "ExtensionParameterDefinitions",
    "ExtensionParameterValue",
    "ExtensionVersion",
    "File",
    "GetUsageType",
    "PatchUser",
    "PatchUserDatabases",
    "Platform",
    "Region",
    "ScaleCluster",
    "Schedule",
    "ScheduleActionType0",
    "ScheduleState",
    "SetAutoUpdatesDatabase",
    "Status",
    "UpdateAllowedIP",
    "UpdateCluster",
    "UpdateDatabase",
    "UpdateProfile",
    "UpdateScheduleCronRule",
    "UpdateScheduleState",
    "UploadFile",
    "Usage",
    "UsageAdditionalPropertyItem",
    "UsageCluster",
    "User",
    "UserDatabase",
    "UserRole",
    "UserStatus",
)



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.user_status import UserStatus
from ..types import (
    UNSET,
    Unset,
)

if TYPE_CHECKING:
    from ..models.user_database import UserDatabase
    from ..models.user_role import UserRole


T = TypeVar("T", bound="User")


@_attrs_define
class User:
    """
    Attributes:
        email (str):
        id (str):
        created_at (datetime.datetime):
        created_by (str):
        status (UserStatus):
        roles (list[UserRole]):
        is_deletable (bool):
        first_name (str | Unset):
        last_name (str | Unset):
        databases (list[UserDatabase] | Unset):
        db_username (str | Unset):
    """

    email: str
    id: str
    created_at: datetime.datetime
    created_by: str
    status: UserStatus
    roles: list[UserRole]
    is_deletable: bool
    first_name: str | Unset = UNSET
    last_name: str | Unset = UNSET
    databases: list[UserDatabase] | Unset = UNSET
    db_username: str | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        email = self.email

        id = self.id

        created_at = self.created_at.isoformat()

        created_by = self.created_by

        status = self.status.value

        roles = []
        for roles_item_data in self.roles:
            roles_item = roles_item_data.to_dict()
            roles.append(roles_item)

        is_deletable = self.is_deletable

        first_name = self.first_name

        last_name = self.last_name

        databases: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.databases, Unset):
            databases = []
            for databases_item_data in self.databases:
                databases_item = databases_item_data.to_dict()
                databases.append(databases_item)

        db_username = self.db_username

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "email": email,
                "id": id,
                "createdAt": created_at,
                "createdBy": created_by,
                "status": status,
                "roles": roles,
                "isDeletable": is_deletable,
            }
        )
        if first_name is not UNSET:
            field_dict["firstName"] = first_name
        if last_name is not UNSET:
            field_dict["lastName"] = last_name
        if databases is not UNSET:
            field_dict["databases"] = databases
        if db_username is not UNSET:
            field_dict["dbUsername"] = db_username

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user_database import UserDatabase
        from ..models.user_role import UserRole

        d = dict(src_dict)
        email = d.pop("email")

        id = d.pop("id")

        created_at = isoparse(d.pop("createdAt"))

        created_by = d.pop("createdBy")

        status = UserStatus(d.pop("status"))

        roles = []
        _roles = d.pop("roles")
        for roles_item_data in _roles:
            roles_item = UserRole.from_dict(roles_item_data)

            roles.append(roles_item)

        is_deletable = d.pop("isDeletable")

        first_name = d.pop("firstName", UNSET)

        last_name = d.pop("lastName", UNSET)

        _databases = d.pop("databases", UNSET)
        databases: list[UserDatabase] | Unset = UNSET
        if _databases is not UNSET:
            databases = []
            for databases_item_data in _databases:
                databases_item = UserDatabase.from_dict(databases_item_data)

                databases.append(databases_item)

        db_username = d.pop("dbUsername", UNSET)

        user = cls(
            email=email,
            id=id,
            created_at=created_at,
            created_by=created_by,
            status=status,
            roles=roles,
            is_deletable=is_deletable,
            first_name=first_name,
            last_name=last_name,
            databases=databases,
            db_username=db_username,
        )

        return user



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

T = TypeVar("T", bound="UpdateAllowedIP")


@_attrs_define
class UpdateAllowedIP:
    """
    Attributes:
        name (str):
        cidr_ip (str):
    """

    name: str
    cidr_ip: str

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        cidr_ip = self.cidr_ip

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "name": name,
                "cidrIp": cidr_ip,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        cidr_ip = d.pop("cidrIp")

        update_allowed_ip = cls(
            name=name,
            cidr_ip=cidr_ip,
        )

        return update_allowed_ip



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

from ..types import (
    UNSET,
    Unset,
)

if TYPE_CHECKING:
    from ..models.patch_user_databases import PatchUserDatabases


T = TypeVar("T", bound="PatchUser")


@_attrs_define
class PatchUser:
    """
    Attributes:
        role_id (str | Unset):
        databases (PatchUserDatabases | Unset):
        db_username (str | Unset):
    """

    role_id: str | Unset = UNSET
    databases: PatchUserDatabases | Unset = UNSET
    db_username: str | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        role_id = self.role_id

        databases: dict[str, Any] | Unset = UNSET
        if not isinstance(self.databases, Unset):
            databases = self.databases.to_dict()

        db_username = self.db_username

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if role_id is not UNSET:
            field_dict["roleID"] = role_id
        if databases is not UNSET:
            field_dict["databases"] = databases
        if db_username is not UNSET:
            field_dict["dbUsername"] = db_username

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.patch_user_databases import PatchUserDatabases

        d = dict(src_dict)
        role_id = d.pop("roleID", UNSET)

        _databases = d.pop("databases", UNSET)
        databases: PatchUserDatabases | Unset
        if isinstance(_databases, Unset):
            databases = UNSET
        else:
            databases = PatchUserDatabases.from_dict(_databases)

        db_username = d.pop("dbUsername", UNSET)

        patch_user = cls(
            role_id=role_id,
            databases=databases,
            db_username=db_username,
        )

        return patch_user



from enum import Enum


class Status(str, Enum):
    CREATING = "creating"
    DELETED = "deleted"
    DELETING = "deleting"
    ERROR = "error"
    MAINTENANCE = "maintenance"
    RUNNING = "running"
    SCALING = "scaling"
    STARTING = "starting"
    STOPPED = "stopped"
    STOPPING = "stopping"
    TOCREATE = "tocreate"
    TODELETE = "todelete"
    TOSCALE = "toscale"
    TOSTART = "tostart"
    TOSTOP = "tostop"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.connection_i_ps import ConnectionIPs


T = TypeVar("T", bound="ClusterConnection")


@_attrs_define
class ClusterConnection:
    """
    Attributes:
        dns (str):
        port (int):
        jdbc (str):
        ips (ConnectionIPs):
        db_username (str):
    """

    dns: str
    port: int
    jdbc: str
    ips: ConnectionIPs
    db_username: str

    def to_dict(self) -> dict[str, Any]:
        dns = self.dns

        port = self.port

        jdbc = self.jdbc

        ips = self.ips.to_dict()

        db_username = self.db_username

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "dns": dns,
                "port": port,
                "jdbc": jdbc,
                "ips": ips,
                "dbUsername": db_username,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.connection_i_ps import ConnectionIPs

        d = dict(src_dict)
        dns = d.pop("dns")

        port = d.pop("port")

        jdbc = d.pop("jdbc")

        ips = ConnectionIPs.from_dict(d.pop("ips"))

        db_username = d.pop("dbUsername")

        cluster_connection = cls(
            dns=dns,
            port=port,
            jdbc=jdbc,
            ips=ips,
            db_username=db_username,
        )

        return cluster_connection



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

T = TypeVar("T", bound="SetAutoUpdatesDatabase")


@_attrs_define
class SetAutoUpdatesDatabase:
    """
    Attributes:
        auto_updates_enabled (bool):
    """

    auto_updates_enabled: bool

    def to_dict(self) -> dict[str, Any]:
        auto_updates_enabled = self.auto_updates_enabled

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "autoUpdatesEnabled": auto_updates_enabled,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        auto_updates_enabled = d.pop("autoUpdatesEnabled")

        set_auto_updates_database = cls(
            auto_updates_enabled=auto_updates_enabled,
        )

        return set_auto_updates_database



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.extension_parameter_value import ExtensionParameterValue


T = TypeVar("T", bound="CreateExtensionInstance")


@_attrs_define
class CreateExtensionInstance:
    """
    Attributes:
        parameter_values (list[ExtensionParameterValue]):
    """

    parameter_values: list[ExtensionParameterValue]

    def to_dict(self) -> dict[str, Any]:
        parameter_values = []
        for parameter_values_item_data in self.parameter_values:
            parameter_values_item = parameter_values_item_data.to_dict()
            parameter_values.append(parameter_values_item)

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "parameterValues": parameter_values,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.extension_parameter_value import ExtensionParameterValue

        d = dict(src_dict)
        parameter_values = []
        _parameter_values = d.pop("parameterValues")
        for parameter_values_item_data in _parameter_values:
            parameter_values_item = ExtensionParameterValue.from_dict(
                parameter_values_item_data
            )

            parameter_values.append(parameter_values_item)

        create_extension_instance = cls(
            parameter_values=parameter_values,
        )

        return create_extension_instance



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

T = TypeVar("T", bound="ClusterActionScale")


@_attrs_define
class ClusterActionScale:
    """
    Attributes:
        cluster_id (str):
        size (str):
    """

    cluster_id: str
    size: str

    def to_dict(self) -> dict[str, Any]:
        cluster_id = self.cluster_id

        size = self.size

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "clusterId": cluster_id,
                "size": size,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        cluster_id = d.pop("clusterId")

        size = d.pop("size")

        cluster_action_scale = cls(
            cluster_id=cluster_id,
            size=size,
        )

        return cluster_action_scale



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="ApiError")


@_attrs_define
class ApiError:
    """
    Attributes:
        status (float):
        message (str):
        request_id (str):
        path (str):
        method (str):
        log_id (str):
        handler (str):
        timestamp (str):
        causes (Any | Unset):
    """

    status: float
    message: str
    request_id: str
    path: str
    method: str
    log_id: str
    handler: str
    timestamp: str
    causes: Any | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        status = self.status

        message = self.message

        request_id = self.request_id

        path = self.path

        method = self.method

        log_id = self.log_id

        handler = self.handler

        timestamp = self.timestamp

        causes = self.causes

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "status": status,
                "message": message,
                "requestId": request_id,
                "path": path,
                "method": method,
                "logId": log_id,
                "handler": handler,
                "timestamp": timestamp,
            }
        )
        if causes is not UNSET:
            field_dict["causes"] = causes

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        status = d.pop("status")

        message = d.pop("message")

        request_id = d.pop("requestId")

        path = d.pop("path")

        method = d.pop("method")

        log_id = d.pop("logId")

        handler = d.pop("handler")

        timestamp = d.pop("timestamp")

        causes = d.pop("causes", UNSET)

        api_error = cls(
            status=status,
            message=message,
            request_id=request_id,
            path=path,
            method=method,
            log_id=log_id,
            handler=handler,
            timestamp=timestamp,
            causes=causes,
        )

        return api_error



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define

T = TypeVar("T", bound="ConnectionIPs")


@_attrs_define
class ConnectionIPs:
    """
    Attributes:
        private (list[str]):
        public (list[str]):
    """

    private: list[str]
    public: list[str]

    def to_dict(self) -> dict[str, Any]:
        private = self.private

        public = self.public

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "private": private,
                "public": public,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        private = cast(list[str], d.pop("private"))

        public = cast(list[str], d.pop("public"))

        connection_i_ps = cls(
            private=private,
            public=public,
        )

        return connection_i_ps



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

from ..types import (
    UNSET,
    Unset,
)

if TYPE_CHECKING:
    from ..models.auto_stop import AutoStop
    from ..models.cluster_settings_update import ClusterSettingsUpdate


T = TypeVar("T", bound="CreateDatabaseInitialCluster")


@_attrs_define
class CreateDatabaseInitialCluster:
    """
    Attributes:
        name (str):
        size (str):
        family (str | Unset):
        auto_stop (AutoStop | Unset):
        settings (ClusterSettingsUpdate | Unset):
    """

    name: str
    size: str
    family: str | Unset = UNSET
    auto_stop: AutoStop | Unset = UNSET
    settings: ClusterSettingsUpdate | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        size = self.size

        family = self.family

        auto_stop: dict[str, Any] | Unset = UNSET
        if not isinstance(self.auto_stop, Unset):
            auto_stop = self.auto_stop.to_dict()

        settings: dict[str, Any] | Unset = UNSET
        if not isinstance(self.settings, Unset):
            settings = self.settings.to_dict()

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "name": name,
                "size": size,
            }
        )
        if family is not UNSET:
            field_dict["family"] = family
        if auto_stop is not UNSET:
            field_dict["autoStop"] = auto_stop
        if settings is not UNSET:
            field_dict["settings"] = settings

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.auto_stop import AutoStop
        from ..models.cluster_settings_update import ClusterSettingsUpdate

        d = dict(src_dict)
        name = d.pop("name")

        size = d.pop("size")

        family = d.pop("family", UNSET)

        _auto_stop = d.pop("autoStop", UNSET)
        auto_stop: AutoStop | Unset
        if isinstance(_auto_stop, Unset):
            auto_stop = UNSET
        else:
            auto_stop = AutoStop.from_dict(_auto_stop)

        _settings = d.pop("settings", UNSET)
        settings: ClusterSettingsUpdate | Unset
        if isinstance(_settings, Unset):
            settings = UNSET
        else:
            settings = ClusterSettingsUpdate.from_dict(_settings)

        create_database_initial_cluster = cls(
            name=name,
            size=size,
            family=family,
            auto_stop=auto_stop,
            settings=settings,
        )

        return create_database_initial_cluster



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="UsageCluster")


@_attrs_define
class UsageCluster:
    """
    Attributes:
        id (str):
        size (str):
        name (str):
        compute (float | Unset):
        out_same_region (float | Unset):
        out_different_region (float | Unset):
        out_internet (float | Unset):
    """

    id: str
    size: str
    name: str
    compute: float | Unset = UNSET
    out_same_region: float | Unset = UNSET
    out_different_region: float | Unset = UNSET
    out_internet: float | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        size = self.size

        name = self.name

        compute = self.compute

        out_same_region = self.out_same_region

        out_different_region = self.out_different_region

        out_internet = self.out_internet

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "size": size,
                "name": name,
            }
        )
        if compute is not UNSET:
            field_dict["compute"] = compute
        if out_same_region is not UNSET:
            field_dict["outSameRegion"] = out_same_region
        if out_different_region is not UNSET:
            field_dict["outDifferentRegion"] = out_different_region
        if out_internet is not UNSET:
            field_dict["outInternet"] = out_internet

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        size = d.pop("size")

        name = d.pop("name")

        compute = d.pop("compute", UNSET)

        out_same_region = d.pop("outSameRegion", UNSET)

        out_different_region = d.pop("outDifferentRegion", UNSET)

        out_internet = d.pop("outInternet", UNSET)

        usage_cluster = cls(
            id=id,
            size=size,
            name=name,
            compute=compute,
            out_same_region=out_same_region,
            out_different_region=out_different_region,
            out_internet=out_internet,
        )

        return usage_cluster



from __future__ import annotations

import getpass
import logging
import time
from collections.abc import Iterable
from contextlib import contextmanager
from datetime import (
    datetime,
    timedelta,
)
from typing import (
    Any,
    TypeVar,
    cast,
)

import tenacity
from tenacity import (
    RetryError,
    TryAgain,
    retry,
)
from tenacity.retry import retry_if_exception_type
from tenacity.stop import stop_after_delay
from tenacity.wait import (
    wait_exponential,
    wait_fixed,
)

from exasol.saas.client import (
    Limits,
    openapi,
)
from exasol.saas.client.openapi.api.clusters import (
    get_cluster_connection,
    list_clusters,
)
from exasol.saas.client.openapi.api.databases import (
    create_database,
    delete_database,
    get_database,
    list_databases,
)
from exasol.saas.client.openapi.api.security import (
    add_allowed_ip,
    delete_allowed_ip,
    list_allowed_i_ps,
)
from exasol.saas.client.openapi.models import (
    ApiError,
    ExasolDatabase,
    Status,
)
from exasol.saas.client.openapi.types import UNSET

LOG = logging.getLogger(__name__)
LOG.setLevel(logging.INFO)
logging.getLogger("httpx").setLevel(logging.WARNING)


def interval_retry(interval: timedelta, timeout: timedelta):
    return tenacity.retry(wait=wait_fixed(interval), stop=stop_after_delay(timeout))


def timestamp_name(project_short_tag: str | None = None) -> str:
    """
    Generates a semi-unique name for a database with the following format:
    - 0-4: number of minutes since the start of the year in hex,
    - 0-5: a semi-random number,
    - provided tag,
    - -username.

    Args:
        project_short_tag: Abbreviation of your project
    """
    now = datetime.now()
    year_start = datetime(now.year, 1, 1)
    minutes_elapsed = int((now - year_start).total_seconds() // 60)
    random_suffix = time.time_ns() % 1048576
    timestamp = f"{minutes_elapsed:05x}{random_suffix:05x}"

    owner = getpass.getuser()
    candidate = f"{timestamp}{project_short_tag or ''}-{owner}"
    return candidate[: Limits.MAX_DATABASE_NAME_LENGTH]


class DatabaseStartupFailure(Exception):
    """
    If a SaaS database instance during startup reports a status other than
    successful.
    """


class DatabaseDeleteTimeout(Exception):
    """
    If deletion of a SaaS database instance was requested but during the
    specified timeout it was still reported in the list of existing databases.
    """


class DatabaseDeleteError(Exception):
    """
    Failed to delete a SaaS database instance.
    """


class OpenApiError(Exception):
    def __init__(self, message: str, error: ApiError | None):
        super().__init__(f"{message}: {error.message}." if error else message)


T = TypeVar("T")


def ensure_type(
    expected: type[T],
    response: T | ApiError | None,
    message: str,
) -> T:
    """
    Ensure the passed response is of the expected type and return it with
    correct type. Otherwise raise an OpenApiError.
    """
    if isinstance(response, expected):
        return cast(T, response)
    api_error = response if isinstance(response, ApiError) else None
    raise OpenApiError(message, api_error)


class InternalError(Exception):
    """
    Internal error during delete with retry.
    """


def create_saas_client(
    host: str,
    pat: str,
    raise_on_unexpected_status: bool = True,
) -> openapi.AuthenticatedClient:
    return openapi.AuthenticatedClient(
        base_url=host,
        token=pat,
        raise_on_unexpected_status=raise_on_unexpected_status,
    )


def _get_database_id(
    account_id: str,
    client: openapi.AuthenticatedClient,
    database_name: str,
) -> str:
    """
    Finds the database id, given the database name.
    """
    dbs = list_databases.sync(account_id, client=client)
    dbs = list(
        filter(
            lambda db: (db.name == database_name)  # type: ignore
            and (db.deleted_at is UNSET)  # type: ignore
            and (db.deleted_by is UNSET),
            dbs,  # type: ignore
        )
    )  # type: ignore
    if not dbs:
        raise RuntimeError(f"SaaS database {database_name} was not found.")
    return dbs[0].id


def get_database_id(
    host: str,
    account_id: str,
    pat: str,
    database_name: str,
) -> str:
    """
    Finds the database id, given the database name.

    Args:
        host:           SaaS service URL.
        account_id:     User account ID
        pat:            Personal Access Token.
        database_name:  Database name.
    """
    with create_saas_client(host, pat) as client:
        return _get_database_id(account_id, client, database_name)


def get_connection_params(
    host: str,
    account_id: str,
    pat: str,
    database_id: str | None = None,
    database_name: str | None = None,
) -> dict[str, Any]:
    """
    Gets the database connection parameters, such as those required by pyexasol:
    - dns
    - user
    - password.
    Returns the parameters in a dictionary that can be used as kwargs when
    creating a connection, like in the code below:

    connection_params = get_connection_params(...)
    connection = pyexasol.connect(**connection_params)

    Args:
        host:           SaaS service URL.
        account_id:     User account ID
        pat:            Personal Access Token.
        database_id:    Database ID, id known.
        database_name:  Database name, in case the id is unknown.
    """

    with create_saas_client(host, pat) as client:
        if not database_id:
            if not database_name:
                raise ValueError(
                    "To get SaaS connection parameters, "
                    "either database name or database id must be provided."
                )
            database_id = _get_database_id(
                account_id, client, database_name=database_name
            )
        clusters = list_clusters.sync(account_id, database_id, client=client)
        cluster_id = next(
            filter(lambda cl: cl.main_cluster, clusters)  # type: ignore
        ).id
        resp = get_cluster_connection.sync(
            account_id, database_id, cluster_id, client=client
        )
        connection = ensure_type(
            openapi.models.ClusterConnection,
            resp,
            "Failed to get the connection data to"
            f" host {host}, account {account_id},"
            f" database with ID {database_id} named {database_name}",
        )
        return {
            "dsn": f"{connection.dns}:{connection.port}",
            "user": connection.db_username,
            "password": pat,
        }


class OpenApiAccess:
    """
    This class is meant to be used only in the context of the API
    generator repository while integration tests in other repositories are
    planned to only use fixture ``saas_database_id()``.
    """

    def __init__(self, client: openapi.AuthenticatedClient, account_id: str):
        self._client = client
        self._account_id = account_id

    def create_database(
        self,
        name: str,
        cluster_size: str = "XS",
        region: str = "eu-central-1",
        idle_time: timedelta | None = None,
    ) -> ExasolDatabase | None:
        def minutes(x: timedelta) -> int:
            return x.seconds // 60

        idle_time = idle_time or Limits.AUTOSTOP_MIN_IDLE_TIME
        cluster_spec = openapi.models.CreateDatabaseInitialCluster(
            name="my-cluster",
            size=cluster_size,
            auto_stop=openapi.models.AutoStop(
                enabled=True,
                idle_time=minutes(idle_time),
            ),
        )
        LOG.info("Creating database %s", name)
        resp = create_database.sync(
            self._account_id,
            client=self._client,
            body=openapi.models.CreateDatabase(
                name=name,
                initial_cluster=cluster_spec,
                provider="aws",
                region=region,
                stream_type="innovation-release",
            ),
        )
        database = ensure_type(
            ExasolDatabase, resp, f"Failed to create database {name}"
        )
        LOG.info("Created database with ID %s", database.id)
        return database

    @contextmanager
    def _ignore_failures(self, ignore: bool = False):
        before = self._client.raise_on_unexpected_status
        self._client.raise_on_unexpected_status = not ignore
        yield self._client
        self._client.raise_on_unexpected_status = before

    def wait_until_deleted(
        self,
        database_id: str,
        timeout: timedelta = timedelta(seconds=1),
        interval: timedelta = timedelta(minutes=1),
    ):
        @interval_retry(interval, timeout)
        def verify_not_listed() -> bool:
            if database_id in self.list_database_ids():
                raise TryAgain
            return True

        try:
            return verify_not_listed()
        except (TryAgain, RetryError) as ex:
            raise DatabaseDeleteTimeout from ex

    def delete_database(
        self,
        database_id: str,
        ignore_failures: bool = False,
        timeout: timedelta = timedelta(minutes=30),
        min_interval: timedelta = timedelta(seconds=1),
        max_interval: timedelta = timedelta(minutes=2),
    ) -> None:
        def is_retry(resp: ApiError) -> bool:
            return (
                resp.status == 400
                and "cluster is not in a proper state" in resp.message
            )

        @retry(
            wait=wait_exponential(
                multiplier=1,
                min=min_interval,
                max=max_interval,
            ),
            stop=stop_after_delay(timeout),
            retry=retry_if_exception_type(TryAgain),
        )
        def delete_with_retry() -> None:
            LOG.info("- Trying to delete ...")
            resp = delete_database.sync(
                self._account_id,
                database_id,
                client=self._client,
            )
            if not isinstance(resp, ApiError):
                # success
                return
            if is_retry(resp):
                raise TryAgain
            raise InternalError(f"HTTP {resp.status}: {resp.message}.")

        LOG.info("Got request to delete database with ID %s", database_id)
        try:
            delete_with_retry()
            LOG.info("Successfully deleted database.")
        except Exception as ex:
            if ignore_failures:
                LOG.warning("Ignoring delete failure: %s", ex)
            else:
                msg = f"Failed to delete database: {ex}"
                LOG.error(msg)
                raise DatabaseDeleteError(msg) from ex

    def list_database_ids(self) -> Iterable[str]:
        resp = list_databases.sync(self._account_id, client=self._client) or []
        # actually list[ExasolDatabase]
        dbs = ensure_type(list, resp, "Failed to list databases")
        return (db.id for db in dbs)

    @contextmanager
    def database(
        self,
        name: str,
        keep: bool = False,
        ignore_delete_failure: bool = False,
        idle_time: timedelta | None = None,
    ):
        db = None
        try:
            db = self.create_database(name, idle_time=idle_time)
            yield db
        finally:
            db_repr = f"{db.name} with ID {db.id}" if db else None
            if not db:
                LOG.warning("Cannot delete database None")
            elif keep:
                LOG.info("Keeping database %s as keep = %s.", db_repr, keep)
            else:
                self.delete_database(db.id, ignore_delete_failure)
                LOG.info("Context assumes database %s as deleted.", db_repr)

    def get_database(
        self,
        database_id: str,
    ) -> ExasolDatabase | None:
        resp = get_database.sync(
            self._account_id,
            database_id,
            client=self._client,
        )
        return ensure_type(
            ExasolDatabase, resp, f"Failed to get database {database_id}"
        )

    def wait_until_running(
        self,
        database_id: str,
        timeout: timedelta = timedelta(minutes=30),
        interval: timedelta = timedelta(minutes=2),
    ):
        success = [Status.RUNNING]

        @interval_retry(interval, timeout)
        def poll_status() -> Status:
            db = self.get_database(database_id)
            status = db.status if db else None
            if status not in success:
                LOG.info("- Database status: %s ...", status)
                raise TryAgain
            return status

        LOG.info("Waiting for database with ID %s to be available:", database_id)
        if poll_status() not in success:
            raise DatabaseStartupFailure()

    def clusters(
        self,
        database_id: str,
    ) -> list[openapi.models.Cluster] | None:
        resp = (
            list_clusters.sync(
                self._account_id,
                database_id,
                client=self._client,
            )
            or []
        )
        # actually list[openapi.models.Cluster]
        return ensure_type(
            list, resp, f"Failed to list clusters of database {database_id}"
        )

    def get_connection(
        self,
        database_id: str,
        cluster_id: str,
    ) -> openapi.models.ClusterConnection | None:
        resp = get_cluster_connection.sync(
            self._account_id,
            database_id,
            cluster_id,
            client=self._client,
        )
        return ensure_type(
            openapi.models.ClusterConnection,
            resp,
            "Failed to retrieve a connection to "
            f"database {database_id} cluster {cluster_id}",
        )

    def list_allowed_ip_ids(self) -> Iterable[str]:
        resp = list_allowed_i_ps.sync(self._account_id, client=self._client) or []
        # actually list[openapi.models.AllowedIP]
        ips = ensure_type(list, resp, "Failed to retrieve the list of allowed ips")
        return (x.id for x in ips)

    def add_allowed_ip(
        self,
        cidr_ip: str = "0.0.0.0/0",
    ) -> openapi.models.AllowedIP | None:
        """
        Suggested values for cidr_ip:
        * 185.17.207.78/32
        * 0.0.0.0/0 = all ipv4
        * ::/0 = all ipv6
        """
        rule = openapi.models.CreateAllowedIP(
            name=timestamp_name(),
            cidr_ip=cidr_ip,
        )
        resp = add_allowed_ip.sync(
            self._account_id,
            client=self._client,
            body=rule,
        )
        return ensure_type(
            openapi.models.AllowedIP,
            resp,
            f"Failed to add allowed IP address {cidr_ip}",
        )

    def delete_allowed_ip(self, id: str, ignore_failures=False) -> Any | None:
        with self._ignore_failures(ignore_failures) as client:
            return delete_allowed_ip.sync(self._account_id, id, client=client)

    @contextmanager
    def allowed_ip(
        self,
        cidr_ip: str = "0.0.0.0/0",
        keep: bool = False,
        ignore_delete_failure: bool = False,
    ):
        ip = None
        try:
            ip = self.add_allowed_ip(cidr_ip)
            yield ip
        finally:
            if ip and not keep:
                self.delete_allowed_ip(ip.id, ignore_delete_failure)



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

T = TypeVar("T", bound="UploadFile")


@_attrs_define
class UploadFile:
    """
    Attributes:
        url (str):
    """

    url: str

    def to_dict(self) -> dict[str, Any]:
        url = self.url

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "url": url,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        url = d.pop("url")

        upload_file = cls(
            url=url,
        )

        return upload_file



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

from ..models.schedule_state import ScheduleState

T = TypeVar("T", bound="UpdateScheduleState")


@_attrs_define
class UpdateScheduleState:
    """
    Attributes:
        state (ScheduleState):
    """

    state: ScheduleState

    def to_dict(self) -> dict[str, Any]:
        state = self.state.value

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "state": state,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        state = ScheduleState(d.pop("state"))

        update_schedule_state = cls(
            state=state,
        )

        return update_schedule_state



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

T = TypeVar("T", bound="UserRole")


@_attrs_define
class UserRole:
    """
    Attributes:
        id (str):
        name (str):
    """

    id: str
    name: str

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        name = d.pop("name")

        user_role = cls(
            id=id,
            name=name,
        )

        return user_role



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.extension_version import ExtensionVersion


T = TypeVar("T", bound="Extension")


@_attrs_define
class Extension:
    """
    Attributes:
        id (str):
        name (str):
        description (str):
        category_id (str):
        installable_versions (list[ExtensionVersion]):
    """

    id: str
    name: str
    description: str
    category_id: str
    installable_versions: list[ExtensionVersion]

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        description = self.description

        category_id = self.category_id

        installable_versions = []
        for installable_versions_item_data in self.installable_versions:
            installable_versions_item = installable_versions_item_data.to_dict()
            installable_versions.append(installable_versions_item)

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "name": name,
                "description": description,
                "categoryId": category_id,
                "installableVersions": installable_versions,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.extension_version import ExtensionVersion

        d = dict(src_dict)
        id = d.pop("id")

        name = d.pop("name")

        description = d.pop("description")

        category_id = d.pop("categoryId")

        installable_versions = []
        _installable_versions = d.pop("installableVersions")
        for installable_versions_item_data in _installable_versions:
            installable_versions_item = ExtensionVersion.from_dict(
                installable_versions_item_data
            )

            installable_versions.append(installable_versions_item)

        extension = cls(
            id=id,
            name=name,
            description=description,
            category_id=category_id,
            installable_versions=installable_versions,
        )

        return extension



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

T = TypeVar("T", bound="DatabaseSettings")


@_attrs_define
class DatabaseSettings:
    """
    Attributes:
        offload_enabled (bool):
        auto_updates_enabled (bool):
        auto_updates_hard_disabled (bool):
        num_nodes (int):
        stream_type (str):
        stream_description (str):
    """

    offload_enabled: bool
    auto_updates_enabled: bool
    auto_updates_hard_disabled: bool
    num_nodes: int
    stream_type: str
    stream_description: str

    def to_dict(self) -> dict[str, Any]:
        offload_enabled = self.offload_enabled

        auto_updates_enabled = self.auto_updates_enabled

        auto_updates_hard_disabled = self.auto_updates_hard_disabled

        num_nodes = self.num_nodes

        stream_type = self.stream_type

        stream_description = self.stream_description

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "offloadEnabled": offload_enabled,
                "autoUpdatesEnabled": auto_updates_enabled,
                "autoUpdatesHardDisabled": auto_updates_hard_disabled,
                "numNodes": num_nodes,
                "streamType": stream_type,
                "streamDescription": stream_description,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        offload_enabled = d.pop("offloadEnabled")

        auto_updates_enabled = d.pop("autoUpdatesEnabled")

        auto_updates_hard_disabled = d.pop("autoUpdatesHardDisabled")

        num_nodes = d.pop("numNodes")

        stream_type = d.pop("streamType")

        stream_description = d.pop("streamDescription")

        database_settings = cls(
            offload_enabled=offload_enabled,
            auto_updates_enabled=auto_updates_enabled,
            auto_updates_hard_disabled=auto_updates_hard_disabled,
            num_nodes=num_nodes,
            stream_type=stream_type,
            stream_description=stream_description,
        )

        return database_settings



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="File")


@_attrs_define
class File:
    """
    Attributes:
        name (str):
        type_ (str):
        path (str):
        last_modified (datetime.datetime):
        size (int | Unset):
        children (list[File] | Unset):
    """

    name: str
    type_: str
    path: str
    last_modified: datetime.datetime
    size: int | Unset = UNSET
    children: list[File] | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        type_ = self.type_

        path = self.path

        last_modified = self.last_modified.isoformat()

        size = self.size

        children: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.children, Unset):
            children = []
            for children_item_data in self.children:
                children_item = children_item_data.to_dict()
                children.append(children_item)

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "name": name,
                "type": type_,
                "path": path,
                "lastModified": last_modified,
            }
        )
        if size is not UNSET:
            field_dict["size"] = size
        if children is not UNSET:
            field_dict["children"] = children

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        type_ = d.pop("type")

        path = d.pop("path")

        last_modified = isoparse(d.pop("lastModified"))

        size = d.pop("size", UNSET)

        _children = d.pop("children", UNSET)
        children: list[File] | Unset = UNSET
        if _children is not UNSET:
            children = []
            for children_item_data in _children:
                children_item = File.from_dict(children_item_data)

                children.append(children_item)

        file = cls(
            name=name,
            type_=type_,
            path=path,
            last_modified=last_modified,
            size=size,
            children=children,
        )

        return file



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

T = TypeVar("T", bound="UpdateProfile")


@_attrs_define
class UpdateProfile:
    """
    Attributes:
        first_name (str):
        last_name (str):
    """

    first_name: str
    last_name: str

    def to_dict(self) -> dict[str, Any]:
        first_name = self.first_name

        last_name = self.last_name

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "firstName": first_name,
                "lastName": last_name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        first_name = d.pop("firstName")

        last_name = d.pop("lastName")

        update_profile = cls(
            first_name=first_name,
            last_name=last_name,
        )

        return update_profile



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define

T = TypeVar("T", bound="PatchUserDatabases")


@_attrs_define
class PatchUserDatabases:
    """
    Attributes:
        delete (list[str]):
        add (list[str]):
    """

    delete: list[str]
    add: list[str]

    def to_dict(self) -> dict[str, Any]:
        delete = self.delete

        add = self.add

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "delete": delete,
                "add": add,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        delete = cast(list[str], d.pop("delete"))

        add = cast(list[str], d.pop("add"))

        patch_user_databases = cls(
            delete=delete,
            add=add,
        )

        return patch_user_databases



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

T = TypeVar("T", bound="AutoStop")


@_attrs_define
class AutoStop:
    """
    Attributes:
        enabled (bool):
        idle_time (int):
    """

    enabled: bool
    idle_time: int

    def to_dict(self) -> dict[str, Any]:
        enabled = self.enabled

        idle_time = self.idle_time

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "enabled": enabled,
                "idleTime": idle_time,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        enabled = d.pop("enabled")

        idle_time = d.pop("idleTime")

        auto_stop = cls(
            enabled=enabled,
            idle_time=idle_time,
        )

        return auto_stop



from http import HTTPStatus
from typing import (
    Any,
    cast,
)
from urllib.parse import quote

import httpx

from ...client import (
    AuthenticatedClient,
    Client,
)
from ...models.api_error import ApiError
from ...models.update_schedule_cron_rule import UpdateScheduleCronRule
from ...types import Response


def _get_kwargs(
    account_id: str,
    database_id: str,
    action_id: str,
    *,
    body: UpdateScheduleCronRule,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "patch",
        "url": "/api/v1/accounts/{account_id}/databases/{database_id}/schedules/{action_id}/cronRule".format(
            account_id=quote(str(account_id), safe=""),
            database_id=quote(str(database_id), safe=""),
            action_id=quote(str(action_id), safe=""),
        ),
    }

    _kwargs["json"] = body.to_dict()

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Any | ApiError:
    if response.status_code == 204:
        response_204 = cast(Any, None)
        return response_204

    response_default = ApiError.from_dict(response.json())

    return response_default


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[Any | ApiError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    account_id: str,
    database_id: str,
    action_id: str,
    *,
    client: AuthenticatedClient,
    body: UpdateScheduleCronRule,
) -> Response[Any | ApiError]:
    """
    Args:
        account_id (str):
        database_id (str):
        action_id (str):
        body (UpdateScheduleCronRule):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any | ApiError]
    """

    kwargs = _get_kwargs(
        account_id=account_id,
        database_id=database_id,
        action_id=action_id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    account_id: str,
    database_id: str,
    action_id: str,
    *,
    client: AuthenticatedClient,
    body: UpdateScheduleCronRule,
) -> Any | ApiError | None:
    """
    Args:
        account_id (str):
        database_id (str):
        action_id (str):
        body (UpdateScheduleCronRule):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Any | ApiError
    """

    return sync_detailed(
        account_id=account_id,
        database_id=database_id,
        action_id=action_id,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    account_id: str,
    database_id: str,
    action_id: str,
    *,
    client: AuthenticatedClient,
    body: UpdateScheduleCronRule,
) -> Response[Any | ApiError]:
    """
    Args:
        account_id (str):
        database_id (str):
        action_id (str):
        body (UpdateScheduleCronRule):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any | ApiError]
    """

    kwargs = _get_kwargs(
        account_id=account_id,
        database_id=database_id,
        action_id=action_id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    account_id: str,
    database_id: str,
    action_id: str,
    *,
    client: AuthenticatedClient,
    body: UpdateScheduleCronRule,
) -> Any | ApiError | None:
    """
    Args:
        account_id (str):
        database_id (str):
        action_id (str):
        body (UpdateScheduleCronRule):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Any | ApiError
    """

    return (
        await asyncio_detailed(
            account_id=account_id,
            database_id=database_id,
            action_id=action_id,
            client=client,
            body=body,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

T = TypeVar("T", bound="ExtensionVersion")


@_attrs_define
class ExtensionVersion:
    """
    Attributes:
        version (str):
        latest (bool):
        deprecated (bool):
        installed (bool):
    """

    version: str
    latest: bool
    deprecated: bool
    installed: bool

    def to_dict(self) -> dict[str, Any]:
        version = self.version

        latest = self.latest

        deprecated = self.deprecated

        installed = self.installed

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "version": version,
                "latest": latest,
                "deprecated": deprecated,
                "installed": installed,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        version = d.pop("version")

        latest = d.pop("latest")

        deprecated = d.pop("deprecated")

        installed = d.pop("installed")

        extension_version = cls(
            version=version,
            latest=latest,
            deprecated=deprecated,
            installed=installed,
        )

        return extension_version



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="ScaleCluster")


@_attrs_define
class ScaleCluster:
    """
    Attributes:
        size (str):
        family (str | Unset):
    """

    size: str
    family: str | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        size = self.size

        family = self.family

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "size": size,
            }
        )
        if family is not UNSET:
            field_dict["family"] = family

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        size = d.pop("size")

        family = d.pop("family", UNSET)

        scale_cluster = cls(
            size=size,
            family=family,
        )

        return scale_cluster



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

from ..models.schedule_action_type_0 import ScheduleActionType0
from ..models.schedule_state import ScheduleState
from ..types import (
    UNSET,
    Unset,
)

if TYPE_CHECKING:
    from ..models.cluster_action_scale import ClusterActionScale
    from ..models.cluster_action_start_stop import ClusterActionStartStop


T = TypeVar("T", bound="Schedule")


@_attrs_define
class Schedule:
    """
    Attributes:
        action (ScheduleActionType0):
        cron_rule (str): cron rule in format: <minute> <hour> <day> <month> <weekday>
        id (str | Unset):
        createdby_id (str | Unset):
        createdby_first_name (str | Unset):
        createdby_last_name (str | Unset):
        cluster_name (str | Unset):
        payload (ClusterActionScale | ClusterActionStartStop | Unset):
        state (ScheduleState | Unset):
    """

    action: ScheduleActionType0
    cron_rule: str
    id: str | Unset = UNSET
    createdby_id: str | Unset = UNSET
    createdby_first_name: str | Unset = UNSET
    createdby_last_name: str | Unset = UNSET
    cluster_name: str | Unset = UNSET
    payload: ClusterActionScale | ClusterActionStartStop | Unset = UNSET
    state: ScheduleState | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        from ..models.cluster_action_scale import ClusterActionScale

        action: str
        if isinstance(self.action, ScheduleActionType0):
            action = self.action.value

        cron_rule = self.cron_rule

        id = self.id

        createdby_id = self.createdby_id

        createdby_first_name = self.createdby_first_name

        createdby_last_name = self.createdby_last_name

        cluster_name = self.cluster_name

        payload: dict[str, Any] | Unset
        if isinstance(self.payload, Unset):
            payload = UNSET
        elif isinstance(self.payload, ClusterActionScale):
            payload = self.payload.to_dict()
        else:
            payload = self.payload.to_dict()

        state: str | Unset = UNSET
        if not isinstance(self.state, Unset):
            state = self.state.value

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "action": action,
                "cronRule": cron_rule,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if createdby_id is not UNSET:
            field_dict["createdbyID"] = createdby_id
        if createdby_first_name is not UNSET:
            field_dict["createdbyFirstName"] = createdby_first_name
        if createdby_last_name is not UNSET:
            field_dict["createdbyLastName"] = createdby_last_name
        if cluster_name is not UNSET:
            field_dict["clusterName"] = cluster_name
        if payload is not UNSET:
            field_dict["payload"] = payload
        if state is not UNSET:
            field_dict["state"] = state

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.cluster_action_scale import ClusterActionScale
        from ..models.cluster_action_start_stop import ClusterActionStartStop

        d = dict(src_dict)

        def _parse_action(data: object) -> ScheduleActionType0:
            if not isinstance(data, str):
                raise TypeError()
            action_type_0 = ScheduleActionType0(data)

            return action_type_0

        action = _parse_action(d.pop("action"))

        cron_rule = d.pop("cronRule")

        id = d.pop("id", UNSET)

        createdby_id = d.pop("createdbyID", UNSET)

        createdby_first_name = d.pop("createdbyFirstName", UNSET)

        createdby_last_name = d.pop("createdbyLastName", UNSET)

        cluster_name = d.pop("clusterName", UNSET)

        def _parse_payload(
            data: object,
        ) -> ClusterActionScale | ClusterActionStartStop | Unset:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                payload_type_0 = ClusterActionScale.from_dict(data)

                return payload_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            payload_type_1 = ClusterActionStartStop.from_dict(data)

            return payload_type_1

        payload = _parse_payload(d.pop("payload", UNSET))

        _state = d.pop("state", UNSET)
        state: ScheduleState | Unset
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = ScheduleState(_state)

        schedule = cls(
            action=action,
            cron_rule=cron_rule,
            id=id,
            createdby_id=createdby_id,
            createdby_first_name=createdby_first_name,
            createdby_last_name=createdby_last_name,
            cluster_name=cluster_name,
            payload=payload,
            state=state,
        )

        return schedule



from __future__ import annotations

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define

T = TypeVar("T", bound="UpdateScheduleCronRule")


@_attrs_define
class UpdateScheduleCronRule:
    """
    Attributes:
        cron_rule (str): cron rule in format: <minute> <hour> <day> <month> <weekday>
    """

    cron_rule: str

    def to_dict(self) -> dict[str, Any]:
        cron_rule = self.cron_rule

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "cronRule": cron_rule,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        cron_rule = d.pop("cronRule")

        update_schedule_cron_rule = cls(
            cron_rule=cron_rule,
        )

        return update_schedule_cron_rule



"""
Package openapi contains the API generated from the JSON definition.
"""

from datetime import (
    timedelta,
)
from typing import Final

from exasol.saas.client.openapi.models.status import Status

SAAS_HOST = "https://cloud.exasol.com"

PROMISING_STATES = [
    Status.CREATING,
    Status.RUNNING,
    Status.STARTING,
    Status.TOCREATE,
    Status.TOSTART,
]


class Limits:
    """
    Constants for Exasol SaaS databases.
    """

    MAX_DATABASE_NAME_LENGTH: Final[int] = 20
    MAX_CLUSTER_NAME_LENGTH: Final[int] = 40
    AUTOSTOP_MIN_IDLE_TIME: Final[timedelta] = timedelta(minutes=15)
    AUTOSTOP_MAX_IDLE_TIME: Final[timedelta] = timedelta(minutes=10000)
    AUTOSTOP_DEFAULT_IDLE_TIME: Final[timedelta] = timedelta(minutes=120)
    # If deleting a database too early, then logging and accounting could be invalid.
    MIN_DATABASE_LIFETIME: Final[timedelta] = timedelta(seconds=30)
